﻿using System;
using System.Collections.Generic;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Problems.MultiObjectiveTestFunctions.Comparators;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {

  /// <summary>
  /// The generational Distance is defined as the mean of a all d[i]^(1/p),
  ///  where d[i] is the minimal distance the ith point of the evaluated front has to any point in the optimal pareto front.   
  /// </summary>
  public class GenerationalDistance : IMultiObjectiveDistance {
    private double p; //dampening Factor

    public GenerationalDistance(double p) {
      this.p = 1 / p;
    }

    public static double GetDistance(IEnumerable<double[]> front, IEnumerable<double[]> optimalFront, double p) {
     return new GenerationalDistance(p).Compare(front,optimalFront);
    }

    public double Compare(IEnumerable<double[]> front, IEnumerable<double[]> optimalFront) {
      //TODO build a kd-tree, sort the array, do someting intelligent here
      if (front == null || optimalFront == null) throw new ArgumentException("Fronts must not be null");
      double sum = 0;
      int c = 0;
     foreach(double[] r in front) {
        sum += Utilities.minDistance(r, optimalFront,true);
        c++;
      }
      if (c == 0) throw new ArgumentException("Fronts must not be empty");
      return Math.Pow(sum, p) /c;
    }

    

  }
}
